@@ -1,5 +1,5 @@
 /* keygen.c  -  key generation regression tests
- *	Copyright (C) 2003 Free Software Foundation, Inc.
+ *	Copyright (C) 2003, 2005 Free Software Foundation, Inc.
  *
  * This file is part of Libgcrypt.
  *
@@ -18,6 +18,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -27,6 +30,7 @@
 
 
 static int verbose;
+static int debug;
 static int error_count;
 
 static void
@@ -91,7 +95,7 @@ check_generated_rsa_key (gcry_sexp_t key, unsigned long expected_e)
           if (verbose)
             print_mpi ("e", e);
         }
-      else if ( gcry_mpi_cmp_ui (e, expected_e)) 
+      else if ( gcry_mpi_cmp_ui (e, expected_e))
         {
           print_mpi ("e", e);
           fail ("public exponent is not %lu\n", expected_e);
@@ -100,7 +104,7 @@ check_generated_rsa_key (gcry_sexp_t key, unsigned long expected_e)
       gcry_mpi_release (e);
       gcry_sexp_release (pkey);
     }
- 
+
   skey = gcry_sexp_find_token (key, "private-key", 0);
   if (!skey)
     fail ("private part missing in return value\n");
@@ -119,10 +123,57 @@ check_rsa_keys (void)
 {
   gcry_sexp_t keyparm, key;
   int rc;
+  int i;
+
+  /* Check that DSA generation works and that it can grok the qbits
+     argument. */
+  if (verbose)
+    fprintf (stderr, "creating 5 1024 bit DSA keys\n");
+  for (i=0; i < 5; i++)
+    {
+      rc = gcry_sexp_new (&keyparm,
+                          "(genkey\n"
+                          " (dsa\n"
+                          "  (nbits 4:1024)\n"
+                          " ))", 0, 1);
+      if (rc)
+        die ("error creating S-expression: %s\n", gpg_strerror (rc));
+      rc = gcry_pk_genkey (&key, keyparm);
+      gcry_sexp_release (keyparm);
+      if (rc)
+        die ("error generating DSA key: %s\n", gpg_strerror (rc));
+      gcry_sexp_release (key);
+      if (verbose)
+        fprintf (stderr, "  done\n");
+    }
 
   if (verbose)
-    fprintf (stderr, "creating 1024 bit RSA key using old interface\n");
-  rc = gcry_sexp_new (&keyparm, 
+    fprintf (stderr, "creating 1536 bit DSA key\n");
+  rc = gcry_sexp_new (&keyparm,
+                      "(genkey\n"
+                      " (dsa\n"
+                      "  (nbits 4:1536)\n"
+                      "  (qbits 3:224)\n"
+                      " ))", 0, 1);
+  if (rc)
+    die ("error creating S-expression: %s\n", gpg_strerror (rc));
+  rc = gcry_pk_genkey (&key, keyparm);
+  gcry_sexp_release (keyparm);
+  if (rc)
+    die ("error generating DSA key: %s\n", gpg_strerror (rc));
+  if (debug)
+    {
+      char buffer[20000];
+      gcry_sexp_sprint (key, GCRYSEXP_FMT_ADVANCED, buffer, sizeof buffer);
+      if (verbose)
+        printf ("=============================\n%s\n"
+                "=============================\n", buffer);
+    }
+  gcry_sexp_release (key);
+
+  if (verbose)
+    fprintf (stderr, "creating 1024 bit RSA key\n");
+  rc = gcry_sexp_new (&keyparm,
                       "(genkey\n"
                       " (rsa\n"
                       "  (nbits 4:1024)\n"
@@ -137,9 +188,10 @@ check_rsa_keys (void)
   check_generated_rsa_key (key, 65537);
   gcry_sexp_release (key);
 
+
   if (verbose)
     fprintf (stderr, "creating 512 bit RSA key with e=257\n");
-  rc = gcry_sexp_new (&keyparm, 
+  rc = gcry_sexp_new (&keyparm,
                       "(genkey\n"
                       " (rsa\n"
                       "  (nbits 3:512)\n"
@@ -157,7 +209,7 @@ check_rsa_keys (void)
 
   if (verbose)
     fprintf (stderr, "creating 512 bit RSA key with default e\n");
-  rc = gcry_sexp_new (&keyparm, 
+  rc = gcry_sexp_new (&keyparm,
                       "(genkey\n"
                       " (rsa\n"
                       "  (nbits 3:512)\n"
@@ -214,11 +266,27 @@ check_nonce (void)
     }
 }
 
+
+static void
+progress_cb (void *cb_data, const char *what, int printchar,
+		  int current, int total)
+{
+  (void)cb_data;
+  (void)what;
+  (void)current;
+  (void)total;
+
+  if (printchar == '\n')
+    fputs ( "<LF>", stdout);
+  else
+    putchar (printchar);
+  fflush (stdout);
+}
+
+
 int
 main (int argc, char **argv)
 {
-  int debug = 0;
-
   if (argc > 1 && !strcmp (argv[1], "--verbose"))
     verbose = 1;
   else if (argc > 1 && !strcmp (argv[1], "--debug"))
@@ -232,10 +300,11 @@ main (int argc, char **argv)
     gcry_control (GCRYCTL_SET_DEBUG_FLAGS, 1u , 0);
   /* No valuable keys are create, so we can speed up our RNG. */
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+  if (verbose)
+    gcry_set_progress_handler ( progress_cb, NULL );
 
   check_rsa_keys ();
   check_nonce ();
-  
+
   return error_count? 1:0;
 }
-
